==> neper-2.0.0.tgz: neper-2.0.0/src/neper_t/net_tess/net_tess3.c <==
/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2013, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "net_tess_.h"

int
net_tess_centroid_coo (struct TESS Tess, double **coo)
{
  int i;

  for (i = 1; i <= Tess.CellQty; i++)
    neut_tess_cell_centroid (Tess, i, coo[i]);

  return 0;
}

int
net_tess_centroid_test (int dim, struct TESS Tess, double** coo,
                        double centroidconv, double *prdistmax)
{
  int i;
  double dist, distmax, tesssize, avrad;

  neut_tess_size (Tess, &tesssize);
  if (dim == 3)
    avrad = pow ((tesssize / Tess.CellQty) / (4 / 3 * M_PI), 0.33333333);
  else if (dim == 2)
  {
    double** bbox = ut_alloc_2d (3, 2);
    neut_tess_bbox (Tess, bbox);
    tesssize /= (bbox[2][1] - bbox[2][0]);
    avrad = pow ((tesssize / Tess.CellQty) / M_PI, 0.5);
    ut_free_2d (bbox, 3);
  }
  else if (dim == 1)
    avrad = .5 * tesssize / Tess.CellQty;
  else
    abort ();

  distmax = 0;
  for (i = 1; i <= Tess.CellQty; i++)
  {
    dist = ut_space_dist (coo[i], Tess.GermCoo[i]);
    distmax = ut_num_max (dist, distmax);
  }

  (*prdistmax) = distmax / avrad;

  /*
  printf ("tesssize = %f\n", tesssize);
  printf ("avrad = %f\n", avrad);
  printf ("distmax = %f\n", distmax);
  printf ("*prdistmax = %f\n", *prdistmax);
  abort ();
  */

  return ((*prdistmax) < centroidconv) ? 0 : 1;
}

int
net_tess_centroid_germupdate (struct TESS Tess, double** coo,
                              double centroidfact, struct GERMSET *pGermSet)
{
  int i, j, id, cooqty = 0;
  int *cooid = ut_alloc_1d_int (3);

  if (!strcmp ((*pGermSet).morpho, "columnar(x)"))
  {
    cooqty = 2;
    cooid[0] = 1;
    cooid[1] = 2;
  }
  else if (!strcmp ((*pGermSet).morpho, "columnar(y)"))
  {
    cooqty = 2;
    cooid[0] = 0;
    cooid[1] = 2;
  }
  else if (!strcmp ((*pGermSet).morpho, "columnar(z)"))
  {
    cooqty = 2;
    cooid[0] = 0;
    cooid[1] = 1;
  }
  else if (!strcmp ((*pGermSet).morpho, "bamboo(x)"))
  {
    cooqty = 1;
    cooid[0] = 0;
  }
  else if (!strcmp ((*pGermSet).morpho, "bamboo(y)"))
  {
    cooqty = 1;
    cooid[0] = 1;
  }
  else if (!strcmp ((*pGermSet).morpho, "bamboo(z)"))
  {
    cooqty = 1;
    cooid[0] = 2;
  }
  else
  {
    cooqty = 3;
    cooid[0] = 0;
    cooid[1] = 1;
    cooid[2] = 2;
  }

  // if 1D/2D tessellation, disabling coordinates
  if ((*pGermSet).Dim < 3)
    cooqty -= ut_array_1d_int_deletencompress (cooid, cooqty, 2, 1);

  if ((*pGermSet).Dim < 2)
    cooqty -= ut_array_1d_int_deletencompress (cooid, cooqty, 1, 1);

  for (i = 1; i <= Tess.CellQty; i++)
    for (j = 0; j < cooqty; j++)
    {
      id = cooid[j];
      (*pGermSet).GermCoo[i][id] =
	(*pGermSet).GermCoo[i][id] * (1 - centroidfact) +
	coo[i][id] * centroidfact;
    }

  ut_free_1d_int (cooid);

  return 0;
}

int
net_tess_set_domain (struct TESS PTess, int poly, int TessId, struct MTESS *pMTess,
	  struct TESS *pTess)
{
  (*pMTess).TessDomVerNb
    = ut_realloc_1d_pint ((*pMTess).TessDomVerNb, TessId + 1);
  (*pMTess).TessDomEdgeNb
    = ut_realloc_1d_pint ((*pMTess).TessDomEdgeNb, TessId + 1);
  (*pMTess).TessDomFaceNb
    = ut_realloc_1d_pint ((*pMTess).TessDomFaceNb, TessId + 1);

  neut_tess_init_domain_poly (pTess, PTess, poly,
			      &((*pMTess).TessDomVerNb[TessId]),
			      &((*pMTess).TessDomEdgeNb[TessId]),
			      &((*pMTess).TessDomFaceNb[TessId]));

  int i, domface, pos;
  for (i = 1; i <= (*pTess).DomFaceQty; i++)
  {
    domface = (*pMTess).TessDomFaceNb[(*pTess).TessId][i];

    pos = ut_array_1d_int_eltpos (PTess.FacePoly[domface], 2, poly);
    if (pos == -1)
      ut_error_reportbug ();

    (*pMTess).DomTessFaceNb[PTess.TessId][domface][pos] = i;
  }

  return 0;
}

int
net_tess_finalize (struct TESS *pTess)
{
  // setting CellTrue and CellBody
  neut_tess_init_celltrue (pTess);
  neut_tess_init_cellbody (pTess);

#ifdef HAVE_DEBUGGING
  if (neut_tess_test (*pTess, 0) != 0 || neut_tess_test_dom (*pTess, 0) != 0)
    ut_print_message (2, 2, "The tessellation is not valid.\n");
#endif

  return 0;
}

==> neper-2.0.1.tgz: neper-2.0.1/src/neper_t/net_tess/net_tess3.c <==
/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2014, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "net_tess_.h"

int
net_tess_centroid_coo (struct TESS Tess, double **coo)
{
  int i;

  for (i = 1; i <= Tess.CellQty; i++)
    neut_tess_cell_centroid (Tess, i, coo[i]);

  return 0;
}

int
net_tess_centroid_test (int dim, struct TESS Tess, double** coo,
                        double centroidconv, double *prdistmax)
{
  int i;
  double dist, distmax, tesssize, avrad;

  neut_tess_size (Tess, &tesssize);
  tesssize /= Tess.CellQty;
  if (dim == 3)
    avrad = pow (tesssize / (4. / 3. * M_PI), 0.33333333);
  else if (dim == 2)
    avrad = sqrt (tesssize / M_PI);
  else if (dim == 1)
    avrad = .5 * tesssize;
  else
    abort ();

  distmax = 0;
  for (i = 1; i <= Tess.CellQty; i++)
  {
    dist = ut_space_dist (coo[i], Tess.GermCoo[i]);
    distmax = ut_num_max (dist, distmax);
  }

  (*prdistmax) = distmax / avrad;

  /*
  printf ("tesssize = %f\n", tesssize);
  printf ("avrad = %f\n", avrad);
  printf ("distmax = %f\n", distmax);
  printf ("*prdistmax = %f\n", *prdistmax);
  abort ();
  */

  return ((*prdistmax) < centroidconv) ? 0 : 1;
}

int
net_tess_centroid_germupdate (struct TESS Tess, double** coo,
                              double centroidfact, struct GERMSET *pGermSet)
{
  int i, j, id, cooqty = 0;
  int *cooid = ut_alloc_1d_int (3);

  if (!strcmp ((*pGermSet).morpho, "columnar(x)"))
  {
    cooqty = 2;
    cooid[0] = 1;
    cooid[1] = 2;
  }
  else if (!strcmp ((*pGermSet).morpho, "columnar(y)"))
  {
    cooqty = 2;
    cooid[0] = 0;
    cooid[1] = 2;
  }
  else if (!strcmp ((*pGermSet).morpho, "columnar(z)"))
  {
    cooqty = 2;
    cooid[0] = 0;
    cooid[1] = 1;
  }
  else if (!strcmp ((*pGermSet).morpho, "bamboo(x)"))
  {
    cooqty = 1;
    cooid[0] = 0;
  }
  else if (!strcmp ((*pGermSet).morpho, "bamboo(y)"))
  {
    cooqty = 1;
    cooid[0] = 1;
  }
  else if (!strcmp ((*pGermSet).morpho, "bamboo(z)"))
  {
    cooqty = 1;
    cooid[0] = 2;
  }
  else
  {
    cooqty = 3;
    cooid[0] = 0;
    cooid[1] = 1;
    cooid[2] = 2;
  }

  // if 1D/2D tessellation, disabling coordinates
  if ((*pGermSet).Dim < 3)
    cooqty -= ut_array_1d_int_deletencompress (cooid, cooqty, 2, 1);

  if ((*pGermSet).Dim < 2)
    cooqty -= ut_array_1d_int_deletencompress (cooid, cooqty, 1, 1);

  for (i = 1; i <= Tess.CellQty; i++)
    for (j = 0; j < cooqty; j++)
    {
      id = cooid[j];
      (*pGermSet).GermCoo[i][id] =
	(*pGermSet).GermCoo[i][id] * (1 - centroidfact) +
	coo[i][id] * centroidfact;
    }

  ut_free_1d_int (cooid);

  return 0;
}

int
net_tess_set_domain (struct TESS PTess, int poly, int TessId, struct MTESS *pMTess,
	  struct TESS *pTess)
{
  (*pMTess).TessDomVerNb
    = ut_realloc_1d_pint ((*pMTess).TessDomVerNb, TessId + 1);
  (*pMTess).TessDomEdgeNb
    = ut_realloc_1d_pint ((*pMTess).TessDomEdgeNb, TessId + 1);
  (*pMTess).TessDomFaceNb
    = ut_realloc_1d_pint ((*pMTess).TessDomFaceNb, TessId + 1);

  neut_tess_init_domain_poly (pTess, PTess, poly,
			      &((*pMTess).TessDomVerNb[TessId]),
			      &((*pMTess).TessDomEdgeNb[TessId]),
			      &((*pMTess).TessDomFaceNb[TessId]));

  int i, domface, pos;
  for (i = 1; i <= (*pTess).DomFaceQty; i++)
  {
    domface = (*pMTess).TessDomFaceNb[(*pTess).TessId][i];

    pos = ut_array_1d_int_eltpos (PTess.FacePoly[domface], 2, poly);
    if (pos == -1)
      ut_error_reportbug ();

    (*pMTess).DomTessFaceNb[PTess.TessId][domface][pos] = i;
  }

  return 0;
}

int
net_tess_finalize (struct TESS *pTess)
{
  // setting CellTrue and CellBody
  neut_tess_init_celltrue (pTess);
  neut_tess_init_cellbody (pTess);

#ifdef HAVE_DEBUGGING
  if ((*pTess).Dim > 1 && neut_tess_test (*pTess, 0) != 0 || neut_tess_test_dom (*pTess, 0) != 0)
    ut_print_message (2, 2, "The tessellation is not valid.\n");
#endif

  return 0;
}
